"""
Download function for related files in the reality model library
"""
import logging
import re
from tqdm import tqdm
import requests

from openxlab.model.clients.openapi_client import OpenapiClient
from openxlab.model.common.constants import endpoint, token, default_metafile_template_name

logger = logging.getLogger("openxlab.model")


def _download(model_repo, file) -> None:
    """
    download model file|meta file|log filee|readme file
    usage: cli & sdk
    """
    try:
        # split params
        username, repository = _split_repo(model_repo)
        client = OpenapiClient(endpoint, token)
        url = client.get_download_url(repository, file)
    except ValueError as e:
        print(f"Error: {e}")
        return
    _download_to_local(url, file)
    print("download model repo:{}, file:{}".format(model_repo, file))


def _download_metafile_template(file=None) -> None:
    """
    download metafile template file
    """
    try:
        # split params
        client = OpenapiClient(endpoint, token)
        url = client.get_metafile_template_download_url()
        print(f"url.{url}")
    except ValueError as e:
        print(f"Error: {e}")
        return
    file = file if file is not None else default_metafile_template_name
    _download_to_local(url, file)
    print("download metafile file:{}".format(file))


def _split_repo(model_repo) -> (str, str):
    """
    Split a full repository name into two separate strings: the username and the repository name.
    """
    # username/repository format check
    pattern = r'^[a-zA-Z0-9]+\/[a-zA-Z0-9\-_]+$'
    if not re.match(pattern, model_repo):
        raise ValueError("The input string must be in the format 'didi12/test-d-1'")

    values = model_repo.split('/')
    return values[0], values[1]


def _download_to_local(url, file) -> None:
    """
    download file to local with progress_bar
    """
    response = requests.get(url, stream=True)

    total_size_in_bytes = int(response.headers.get('content-length', 0))
    block_size = 1024
    progress_bar = tqdm(total=total_size_in_bytes, unit='iB', unit_scale=True)

    with open(file, 'wb') as file:
        for data in response.iter_content(block_size):
            progress_bar.update(len(data))
            file.write(data)
    progress_bar.close()

# def _get_download_url(url, payload) -> dict:
#     """
#     获取文件下载路径
#     """
#     response = requests.post(url, json=payload)
#     response.raise_for_status()  # 抛出异常如果状态码不是200
#     response_dict = response.json()  # 转换为Python字典格式
#     response_dto = ReturnDto.from_camel_case(response_dict)
#     if response_dto.msg_code != '10000':
#         raise ValueError(f"Get download url error:{response_dto.msg}, traceId:{response_dto.trace_id}")
#     return response_dto.data['url']



"""
create model repository-cli
"""
from openxlab.types.command_type import *


class Create(BaseCommand):
    """create"""

    def get_name(self) -> str:
        return "create"

    def add_arguments(self, parser: ArgumentParser) -> None:
        pass

    def take_action(self, parsed_args: Namespace) -> int:
        print("create something")
        return 0



from openxlab.types.command_type import *
from openxlab.model.commands.download import Download
from openxlab.model.commands.upload import Upload
from openxlab.model.commands.init import Init
from openxlab.model.commands.create import Create


class Model(BaseCommand):
    """model"""

    sub_command_list = [Upload, Download, Init, Create]

    def get_name(self) -> str:
        return "model"

    def add_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument(
            "--foo",
            type=str,
            help=(
                "this is an argument for test"
            ),
        )

    def take_action(self, parsed_args: Namespace) -> int:
        pass



"""
download model file|meta file|log file|readme file-cli
"""
from openxlab.types.command_type import *
from openxlab.model.handler import download


class Download(BaseCommand):
    """download"""

    def get_name(self) -> str:
        return "download"

    def add_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument('-m', '--model-repo', required=True,
                            help='model repository address. format:username/repository.')
        parser.add_argument('-f', '--file', required=True,
                            help='target file to be download.')

    def take_action(self, parsed_args: Namespace) -> int:
        download(parsed_args.model_repo, parsed_args.file)
        # print("download something")
        return 0



"""
delete model repository-cli
"""
from openxlab.types.command_type import *


class Remove(BaseCommand):
    """remove"""

    def get_name(self) -> str:
        return "remove"

    def add_arguments(self, parser: ArgumentParser) -> None:
        pass

    def take_action(self, parsed_args: Namespace) -> int:
        print("model repo remove")
        return 0



"""
model repository init-cli
"""
from openxlab.types.command_type import *
from openxlab.model.handler import download_metafile_template


class Init(BaseCommand):
    """init"""

    def get_name(self) -> str:
        return "init"

    def add_arguments(self, parser: ArgumentParser) -> None:
        pass

    def take_action(self, parsed_args: Namespace) -> int:
        download_metafile_template()
        return 0



"""
get model list of repository-cli
"""
from openxlab.types.command_type import *


class List(BaseCommand):
    """list"""

    def get_name(self) -> str:
        return "list"

    def add_arguments(self, parser: ArgumentParser) -> None:
        pass

    def take_action(self, parsed_args: Namespace) -> int:
        print("model list")
        return 0



"""
setting model repository visibility-cli
"""
from openxlab.types.command_type import *


class Visibility(BaseCommand):
    """visibility"""

    def get_name(self) -> str:
        return "visibility"

    def add_arguments(self, parser: ArgumentParser) -> None:
        pass

    def take_action(self, parsed_args: Namespace) -> int:
        print("change visibility")
        return 0



import requests

from openxlab.model.common.constants import paths
from openxlab.model.common.response_dto import ReturnDto
import os


class OpenapiClient(object):
    def __init__(self, endpoint, token):
        self.endpoint = endpoint
        self.token = token

    # def get_dataset_files(self, dataset_name:str, prefix:str):
    def get_download_url(self, repository, file):
        """
        get file(model file|meta file|log file|readme file) download url
        """
        client_from = os.environ.get('CLIENT_FROM', '0')
        payload = {"repositoryName": repository, "fileName": file, "clientFrom": client_from}
        path = paths['file_download_path']
        response_dto = self.http_post_response_dto(path, payload)
        print(f"response_dto.data:{response_dto.data}")
        if response_dto.msg_code != '10000':
            raise ValueError(f"Get download url error:{response_dto.msg}, traceId:{response_dto.trace_id}")
        if response_dto.data['msgCode'] != '10000':
            raise ValueError(f"Get download url error:{response_dto.data['msg']}, "
                             f"traceId:{response_dto.data['traceId']}")
        return response_dto.data['data']['url']

    def get_metafile_template_download_url(self, file=None):
        """
        get metafile template download url
        """
        payload = {}
        path = paths['meta_file_template_download_path']
        response_dto = self.http_post_response_dto(path, payload)
        if response_dto.msg_code != '10000':
            raise ValueError(f"Get download url error:{response_dto.msg}, traceId:{response_dto.trace_id}")
        if response_dto.data['msgCode'] != '10000':
            raise ValueError(f"Get download url error:{response_dto.data['msg']}, "
                             f"traceId:{response_dto.data['traceId']}")
        return response_dto.data['data']['url']

    def http_post_response_dto(self, path, payload):
        headers = self.http_common_header()
        print(f"endpoint:{self.endpoint}{path}")
        response = requests.post(f"{self.endpoint}{path}", json=payload, headers=headers)
        response.raise_for_status()
        response_dict = response.json()
        response_dto = ReturnDto.from_camel_case(response_dict)
        return response_dto

    def http_common_header(self):
        header_dict = {
            "Content-Type": "application/json",
            "accept": "application/json",
            "Authorization": f"Bearer {self.token}"
        }
        return header_dict



"""
model module constant define
"""

model_openapi_url_prefix_dev = "https://dev.openapi.openxxlab.com/api/v1"
model_openapi_url_prefix_prod = "https://openapi.openxlab.org.cn/api/v1"
model_url_prefix_dev = "http://localhost:10019"
# temp token
token = "eyJ0eXBlIjoiSldUIiwiYWxnIjoiSFM1MTIifQ.eyJqdGkiOiI1OTcwOTEiLCJyb2wiOiJST0xFX0FETUlOIiwiaXNzIjoiT3BlblhMYWIiLCJpYXQiOjE2ODE5Njg4NzMsInBob25lIjoiIiwiYWsiOiI2cHFnOXprNmRteG9rZ2JnbHZvayIsImVtYWlsIjoiZG9uZ3hpYW96aHVhbmdAcGpsYWIub3JnLmNuIiwiZXhwIjoxNzEzNTA0ODczfQ.eFl8ZH9tDp-pcjY3wz6PeNBarJwhVx90qQ3h82Qvpf0hrcrdrQSBcI8AmEk2TZFpeViC6HBtXRxxGp2YLK1XkA"
endpoint = model_openapi_url_prefix_dev
paths = {
    'file_download_path': '/model-center/api/v1/cli/repository/getFileDownloadUrl',
    'meta_file_template_download_path': '/model-center/api/v1/cli/repository/getMetafileTemplateUrl',
}
default_metafile_template_name = 'metafile_template.yaml'



"""
upload model file|meta file|log file|readme file-cli
"""
from openxlab.types.command_type import *


class Upload(BaseCommand):
    """upload"""

    def get_name(self) -> str:
        return "upload"

    def add_arguments(self, parser: ArgumentParser) -> None:
        pass

    def take_action(self, parsed_args: Namespace) -> int:
        print("upload something")
        return 0
